/*
	This file is derived from modes.l from the MPlayer project
	Modifications (C) 2004 RB

	The origial copyright of the file follows below

        $Id: config.l 59 2004-07-30 21:47:15Z rsnel $
*/
/*
 *  Linux Frame Buffer Device Configuration
 *
 *   Copyright 1995-1998 by Geert Uytterhoeven
 *                     (Geert.Uytterhoeven@cs.kuleuven.ac.be)
 *
 *  --------------------------------------------------------------------------
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of the Linux
 *  distribution for more details.
 */

%{

#define YYSTYPE	long

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include "parser.h"
#include "config.tab.h"

/* To avoid the warning yyunput defined but not used */
#define YY_NO_UNPUT


	   
struct keyword_s {
    const char *name;
    int token;
    int val;
};

static const struct keyword_s keywords[] = {
    { "polx", POLX,0},
    { "poly", POLY,0},
    { "poltresh", POLTRESH,0},
    { "polnorm", POLNORM,0},
    { "bright_adjust", BRIGHT_ADJUST,0},
    { "center", CENTER,0},
    { "refpoint", REFPOINT,0},
    { "locktresh", LOCKTRESH,0},
    { "chan", CHAN,0},
    { "freq", FREQ,0},
    { "global", GLOBAL,0},
    { "filter", FILTER,0},
    { "channel", CHANNEL,0},
    { "composite", COMPOSITE,0},
    { "end", END,0},
    { "m1", REFVAL,M1},
    { "m2", REFVAL,M2},
    { "", -1,0}
};

int line = 1;
double dbl;

void yyerror(const char *s)
{
    parse_error(s);
}


int yywrap(void)
{
    return 1;
}

static int find_keyword(const char *s)
{
	int i;

	for (i = 0; keywords[i].token >= 0; i++)
		if(!strcasecmp(s, keywords[i].name)) {
			yylval = keywords[i].val;
			return keywords[i].token;
		}

	parse_error("Unknown keyword %s",s);	
	return 0;
}

%}
%option nounput

keyword [a-zA-Z][a-zA-Z0-9_]*
number  [0-9]*
dblval  [0-9]+\.[0-9]+
string  \"[^\"\n]*\"
comment	\#([^\n]*)
space	[ \t]+
junk	.

%%

{keyword}   {
                return find_keyword(yytext);
            }

{number}    {
                yylval = strtoul(yytext, NULL, 0);
                return INTEGER;
            }

{dblval}    {
		dbl=atof(yytext);
		yylval = (unsigned long)&dbl;
		return DOUBLE;	
            }

{string}    {
		yytext[yyleng-1]='\0';
                yylval = (unsigned long)(yytext+1);
                return STRING;
            }

{comment}$  break;

{space}	    break;

\n	    {
		line++;
		break;
	    }

{junk}	    {
		parse_error("Invalid token `%s'",yytext);
	    }

%%

