/* parser.c - parser module of cc2y4m
 *
 * Copyright (C) 2004 RB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: parser.c 59 2004-07-30 21:47:15Z rsnel $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "cc2y4m.h"

#include "parser.h"
#include "frequencies.h"

extern int line;
char *file;

struct channel_s *first_channel = NULL, *last_channel = NULL;
extern const struct lock_poldetect lock_defaults[];
struct lock_poldetect global_par = { -1, -1, -1, -1, -1, -1, -1, NULL, -1};

double lookup_frequency(char *f)
{
	int i;
	for(i=0;i<sizeof(europe_west)/sizeof(struct CHANLIST);i++)
		if(!strcmp(f,europe_west[i].chan))
			return europe_west[i].freq/1000.0;
	return 0.0;
}

struct channel_s *parse_file(char *fich)
{
	struct channel_s *ptr;
	const struct lock_poldetect *defpars;

	file=fich;
	yyin=fopen(file,"r");
	if(!yyin) return NULL; 
	yyparse();

	/* Fill in the missing values with defaults from lock_defaults */

	ptr=first_channel;
	for(;;) {

		if(ptr->p.refpoint==NULL) {
			ptr->p.refpoint=refpoint_m1;
			defpars=&lock_defaults[0];
		}
		else if(ptr->p.refpoint==refpoint_m1)
			defpars=&lock_defaults[0];
		else if(ptr->p.refpoint==refpoint_m2)
			defpars=&lock_defaults[1];
		else {
			fprintf(stderr,"parse_file internal error\n");
			exit(1);
		}

		if(ptr->freq == -2)
			parse_error("No channel or freq specified for %s",
				ptr->name);
		if(ptr->p.filter==-1) ptr->p.filter=defpars->filter;
		if(ptr->p.polx==-1) ptr->p.polx=defpars->polx;
		if(ptr->p.poly==-1) ptr->p.poly=defpars->poly;
		if(ptr->p.poltresh==-1) ptr->p.poltresh=defpars->poltresh;
		if(ptr->p.polnorm==-1) ptr->p.polnorm=defpars->polnorm;
		if(ptr->p.bright_adjust==-1)
			ptr->p.bright_adjust=defpars->bright_adjust;
		if(ptr->p.center==-1) ptr->p.center=defpars->center;
		if(ptr->p.locktresh==-1) ptr->p.locktresh=defpars->locktresh;

#ifdef DEBUG_PARSER
		printf("%s\n",ptr->name);
		if(ptr->p.refpoint==refpoint_m1) 
			printf("    refpoint:  m1\n");
		else if(ptr->p.refpoint==refpoint_m2)
			printf("    refpoint:  m2\n");
		printf("    freq:      %ld\n",ptr->freq);
		printf("    polx:      %d\n",ptr->p.polx);
		printf("    poly:      %d\n",ptr->p.poly);
		printf("    poltresh:  %d\n",ptr->p.poltresh);
		printf("    polnorm:   %d\n",ptr->p.polnorm);
		printf("    bright:    %d\n",ptr->p.bright_adjust);
		printf("    center:    %d\n",ptr->p.center);
		printf("    locktresh: %d\n",ptr->p.locktresh);
#endif

		ptr=ptr->next;
		if(ptr==first_channel) break;
	}
	return first_channel;
		
}

void parse_error(const char *fmt, ...)
{
        va_list va;

        fflush(stdout);
        fprintf(stderr,"\n%s:%d: ",file,line);
        va_start(va, fmt);
        vfprintf(stderr, fmt, va);
        fprintf(stderr,"\n");
        va_end(va);
        exit(1);
}

void new_channel(struct channel_s *chan)
{
	struct channel_s *ptr;

	ptr=malloc(sizeof(struct channel_s));
	if(ptr==NULL) parse_error("Out of memory");
	if(first_channel==NULL) last_channel=first_channel=ptr;
	ptr->prev=last_channel;
	ptr->next=first_channel;
	last_channel->next=ptr;
	first_channel->prev=ptr;
	last_channel=ptr;
	/* put in the global parameters */
	ptr->p=global_par;
	/* copy the params defined in the config file */
	if(chan->p.polx!=-1) ptr->p.polx=chan->p.polx;
	if(chan->p.poly!=-1) ptr->p.poly=chan->p.poly;
	if(chan->p.poltresh!=-1) ptr->p.poltresh=chan->p.poltresh;
	if(chan->p.polnorm!=-1) ptr->p.polnorm=chan->p.polnorm;
	if(chan->p.bright_adjust!=-1)
		ptr->p.bright_adjust=chan->p.bright_adjust;
	if(chan->p.center!=-1) ptr->p.center=chan->p.center;
	if(chan->p.refpoint!=NULL) ptr->p.refpoint=chan->p.refpoint;
	if(chan->p.locktresh!=-1) ptr->p.locktresh=chan->p.locktresh;
	ptr->name=chan->name;
	ptr->freq=chan->freq;
}
